/* GTK - The GIMP Toolkit
 * Copyright (C) 1995-1997 Peter Mattis, Spencer Kimball and Josh MacDonald
 *
 * This library is free software; you can redistribute it and/or
 * modify it under the terms of the GNU Lesser General Public
 * License as published by the Free Software Foundation; either
 * version 2 of the License, or (at your option) any later version.
 *
 * This library is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 * Lesser General Public License for more details.
 *
 * You should have received a copy of the GNU Lesser General Public
 * License along with this library; if not, write to the
 * Free Software Foundation, Inc., 59 Temple Place - Suite 330,
 * Boston, MA 02111-1307, USA.
 */

/*
 * Modified by the GTK+ Team and others 1997-2000.  See the AUTHORS
 * file for a list of people on the GTK+ Team.  See the ChangeLog
 * files for a list of changes.  These files are distributed with
 * GTK+ at ftp://ftp.gtk.org/pub/gtk/.
 */

#include <stdarg.h>
#include <string.h>

#include <gobject/gparamspecs.h>
#include <gobject/gsignal.h>

#include "gaccessible.h"
#include "gaccessibleaction.h"
#include "gaccessiblecomponent.h"
#include "gaccessibleeditabletext.h"
#include "gaccessiblehypertext.h"
#include "gaccessibleicon.h"
#include "gaccessibleselection.h"
#include "gaccessibletable.h"
#include "gaccessibletext.h"
#include "gaccessiblevalue.h"

/* New GObject properties registered by Gaccessible */
enum
{
  PROP_0,  /* gobject convention */

  PROP_NAME,
  PROP_DESCRIPTION,
  PROP_PARENT,      /* ancestry has changed */
  PROP_CHILD,       /* children have changed in number or identity */
  PROP_ROLE,        /* GaccessibleRole has changed  */
  PROP_STATE        /* GaccessibleState has changed */
};


typedef struct
{
  GObject   target;
  guint     relationship;
} _GaccessibleRelation;

typedef struct
{
  GList     *relations;
} _GaccessibleRelationSet;


enum {
  CHILDREN_CHANGED,
  LAST_SIGNAL
};

static void gaccessible_class_init (GaccessibleClass *klass);
static void gaccessible_init (Gaccessible      *accessible,
                              GaccessibleClass *klass);

static gchar* state_names[NUM_POSSIBLE_STATES];

static guint gaccessible_signals[LAST_SIGNAL] = { 0, };

static gpointer  parent_class = NULL;

GType
gaccessible_get_type (void)
{
  static GType accessible_type = 0;

  if (!accessible_type)
    {
      GTypeInfo typeInfo =
      {
        sizeof (GaccessibleClass),
        (GBaseInitFunc) NULL,
        (GBaseFinalizeFunc) NULL,
        (GClassInitFunc) gaccessible_class_init,
        (GClassFinalizeFunc) NULL,
        NULL,
        sizeof (Gaccessible),
        0,
        (GInstanceInitFunc) gaccessible_init,
      } ;
      accessible_type = g_type_register_static (G_TYPE_OBJECT, "Gaccessible", &typeInfo, 0) ;
    }
  return accessible_type ;
}

static void
gaccessible_class_init (GaccessibleClass *klass)
{
  GObjectClass *gobject_class = G_OBJECT_CLASS (klass);

  parent_class = g_type_class_ref (G_TYPE_OBJECT);

#if 0
  gobject_class->finalize = gaccessible_class_finalize;
#endif

  state_names[STATE_ARMED]           = "armed";
  state_names[STATE_BUSY]            = "busy";
  state_names[STATE_CHECKED]         = "checked";
  state_names[STATE_COLLAPSED]       = "collapsed";
  state_names[STATE_EDITABLE]        = "editable";
  state_names[STATE_ENABLED]         = "enabled";
  state_names[STATE_EXPANDABLE]      = "expandable";
  state_names[STATE_EXPANDED]        = "expanded";
  state_names[STATE_FOCUSABLE]       = "focusable";
  state_names[STATE_FOCUSED]         = "focused";
  state_names[STATE_HORIZONTAL]      = "horizontal";
  state_names[STATE_ICONIFIED]       = "iconified";
  state_names[STATE_MODAL]           = "modal";
  state_names[STATE_MULTI_LINE]      = "multi-line";
  state_names[STATE_MULTISELECTABLE] = "multiselectable";
  state_names[STATE_OPAQUE]          = "opaque";
  state_names[STATE_PRESSED]         = "pressed";
  state_names[STATE_RESIZABLE]       = "resizeable";
  state_names[STATE_SELECTABLE]      = "selectable";
  state_names[STATE_SELECTED]        = "selected";
  state_names[STATE_SHOWING]         = "showing";
  state_names[STATE_SINGLE_LINE]     = "single-line";
  state_names[STATE_TRANSIENT]       = "transient";
  state_names[STATE_VERTICAL]        = "vertical";
  state_names[STATE_VISIBLE]         = "visible";

  klass->children_changed = NULL;

#if 0
  /* ??? Would like to understand the GFlags system a little better before doing this
   *
   * possible alternative ?
   *   g_flags_register_static ("accessible_state",
   *                          accessible_state_flags);
   */
  /* register some properties - these could be change signals instead */
  g_object_class_install_property (gobject_class,
                                   PROP_NAME,
                                   g_param_spec_ccallback ("accessible_name", "Accessible Name",
                                                           "Object instance\'s name formatted for "
                                                           "assistive technology access",
                                                           G_PARAM_READWRITE));
  g_object_class_install_property (gobject_class,
                                   PROP_DESCRIPTION,
                                   g_param_spec_ccallback ("accessible_description", "Accessible Description",
                                                           "Description of an object, formatted for "
                                                           "assistive technology access",
                                                           G_PARAM_READWRITE));
  g_object_class_install_property (gobject_class,
                                   PROP_PARENT,
                                   g_param_spec_ccallback ("accessible_parent", "Accessible Parent",
                                                           "First accessible ancestor (container or object) "
                                                           "which this object is a descendant of",
                                                           G_PARAM_READWRITE));

  /* this may be inappropriate as a property since we don't want to enumerate the children until asked,
   * though we _do_ need to expose change notifications when children are added, deleted, or replaced.
   *  ( see below, we register a new signal for this reason )
   *
   *   g_object_class_install_property (gobject_class,
   *                                    PROP_CHILD,
   *                                    g_param_spec_ccallback ("accessible_children", "Accessible Children",
   *                                                            "A list of Gaccessible instances which are "
   *                                                            "children of this object",
   *                                                            G_PARAM_READ));
   */

  g_object_class_install_property (gobject_class,
                                   PROP_ROLE,
                                   g_param_spec_ccallback ("accessible_role", "Accessible Role",
                                                           "The user-interface role of this object",
                                                           G_PARAM_READWRITE));
  g_object_class_install_property (gobject_class,
                                   PROP_STATE,
                                   g_param_spec_ccallback ("accessible_state", "Accessible State",
                                                           "The current state of this object "
                                                           "or its UI component",
                                                           G_PARAM_READWRITE));
  gaccessible_signals[CHILDREN_CHANGED] =
    g_signal_newc ("accessible_children_changed",
                   G_TYPE_FROM_CLASS (klass),
                   G_SIGNAL_RUN_LAST,
                   G_STRUCT_OFFSET (GaccessibleClass, children_changed), /* still need to declare and define this func */
                   NULL,
#if 0
This function does not exist 
                   g_cclosure_marshal_VOID__UINT_UINT_POINTER,
                   G_TYPE_NONE,
                   3, G_TYPE_UINT, G_TYPE_UINT, TYPE_GACCESSIBLE);
#endif
                   g_cclosure_marshal_VOID__UINT_POINTER,
                   G_TYPE_NONE,
                   2, G_TYPE_UINT, TYPE_GACCESSIBLE);
#endif
}

static void
gaccessible_init (Gaccessible      *accessible,
                  GaccessibleClass *klass)
{
}

GType
gaccessible_interface_get_type (void)
{
  static guint iface_type = 0;

  if (!iface_type)
    {
      GTypeInfo typeInfo =
      {
        sizeof (GaccessibleInterface),
        (GBaseInitFunc) NULL,
        (GBaseFinalizeFunc) NULL,
      } ;

      iface_type = g_type_register_static (G_TYPE_INTERFACE, "gaccessible_interface", &typeInfo, 0) ;
    }

  return iface_type ;
}

gchar*
gaccessible_get_name (Gaccessible *accessible)
{
  g_return_val_if_fail ((accessible != NULL), NULL);
  g_return_val_if_fail (G_IS_GACCESSIBLE (accessible), NULL);
  return GACCESSIBLE (accessible)->name;
}

gchar*
gaccessible_get_description (Gaccessible *accessible)
{
  g_return_val_if_fail ((accessible != NULL), NULL);
  g_return_val_if_fail (G_IS_GACCESSIBLE (accessible), NULL);
  return GACCESSIBLE (accessible)->description;
}

Gaccessible*
gaccessible_get_parent (Gaccessible *accessible)
{
  Gaccessible *parent;

  g_return_val_if_fail ((accessible != NULL), NULL);
  g_return_val_if_fail (G_IS_GACCESSIBLE (accessible), NULL);

  parent = accessible->parent ;
  g_return_val_if_fail (G_IS_GACCESSIBLE (parent), NULL);

  return parent;
}

guint
gaccessible_get_accessible_children_count (Gaccessible *accessible)
{
  GaccessibleClass *klass;

  g_return_val_if_fail ((accessible != NULL), 0);
  g_return_val_if_fail (G_IS_GACCESSIBLE (accessible), 0);

  klass = GACCESSIBLE_GET_CLASS (accessible);
  g_return_val_if_fail ((klass->get_child_count != NULL), 0);

  return (klass->get_child_count) (accessible);
}

Gaccessible*
gaccessible_ref_accessible_child (Gaccessible *accessible,
                                  guint       i)
{
  GaccessibleClass *klass;

  g_return_val_if_fail ((accessible != NULL), NULL);
  g_return_val_if_fail (G_IS_GACCESSIBLE (accessible), NULL);

  klass = GACCESSIBLE_GET_CLASS (accessible);
  g_return_val_if_fail ((klass->ref_child != NULL), NULL);

  return (klass->ref_child) (accessible, i);
}

GaccessibleRelationSet*
gaccessible_get_relation_set (Gaccessible *accessible)
{
  GaccessibleClass *klass;

  g_return_val_if_fail ((accessible != NULL), NULL);
  g_return_val_if_fail (G_IS_GACCESSIBLE (accessible), NULL);

  klass = GACCESSIBLE_GET_CLASS (accessible);
#if 0
 This function does not exist; should it?
  g_return_val_if_fail ((klass->get_relation_set != NULL), NULL);

  return (klass->get_relation_set) (accessible);
#endif
  return NULL;
}

GaccessibleRole
gaccessible_get_role (Gaccessible *accessible) {
  g_return_val_if_fail ((accessible != NULL), 0);
  g_return_val_if_fail (G_IS_GACCESSIBLE (accessible), 0);

  return (GACCESSIBLE (accessible))->role;
}

GaccessibleState*
gaccessible_get_state (Gaccessible *accessible) {
  g_return_val_if_fail ((accessible != NULL), NULL);
  g_return_val_if_fail (G_IS_GACCESSIBLE (accessible), NULL);

  return (GACCESSIBLE (accessible))->state;
}

guint
gaccessible_get_index_in_parent (Gaccessible *accessible)
{
  g_return_val_if_fail ((accessible != NULL), 0);
  g_return_val_if_fail (G_IS_GACCESSIBLE (accessible), 0);
#if 0
  This function does not exist. Should it?
  g_return_val_if_fail ((GACCESSIBLE (accessible)->get_index_in_parent != NULL), 0);

  return (* get_index_in_parent) (accessible);
#endif
  return 0;
}

void
gaccessible_set_name (Gaccessible *accessible,
                      const gchar *name)
{
  g_return_if_fail ((accessible != NULL));
  g_return_if_fail (G_IS_GACCESSIBLE (accessible));
  g_return_if_fail ((name != NULL));

  (GACCESSIBLE (accessible))->name = g_strdup(name);
}

void
gaccessible_set_description (Gaccessible *accessible,
                             const gchar *description)
{
  g_return_if_fail ((accessible != NULL));
  g_return_if_fail (G_IS_GACCESSIBLE (accessible));
  g_return_if_fail ((description != NULL));

  /* TODO: Check memory management policy regarding const gchar* params */
  (GACCESSIBLE (accessible))->description = g_strdup(description);
  /* TODO: Property notification */
}

void
gaccessible_set_parent (Gaccessible *accessible,
                        Gaccessible *parent)
{
  g_return_if_fail ((accessible != NULL));
  g_return_if_fail (G_IS_GACCESSIBLE (accessible));

  /* TODO: Check memory management policy regarding const gchar* params */
  (GACCESSIBLE (accessible))->parent = parent;
  /* TODO: Property notification. */
}

guint
gaccessible_connect_property_change_handler (Gaccessible *accessible,
                                             GaccessiblePropertyChangeHandler *handler)
{
  g_return_val_if_fail ((accessible != NULL), 0);
  g_return_val_if_fail (G_IS_GACCESSIBLE (accessible), 0);

  /* TODO: type checking for handler */
  return g_signal_connect_closure( GACCESSIBLE (accessible), "properties_changed",
                                  (GClosure* ) handler, TRUE);
}

void
gaccessible_remove_property_change_handler  (Gaccessible *accessible,
                                             guint       handler_id)
{
  g_return_if_fail ((accessible != NULL));
  g_return_if_fail (G_IS_GACCESSIBLE (accessible));

  g_signal_handler_disconnect (GACCESSIBLE (accessible), handler_id);
}

gchar*
gaccessible_state_mask_get_name (GaccessibleStateMask state)
{
  gint n;

  g_return_val_if_fail (state, NULL);

  for (n=0; n<NUM_POSSIBLE_STATES; ++n)
    {
      /* fall through and return null if multiple bits are set */
      if (state == (1 << n)) return state_names[n];
    }

  return NULL;
}

GaccessibleStateMask
gaccessible_state_mask_for_name (gchar *name)
{
  gint i;

  g_return_val_if_fail ((strlen(name)>0), 0);
  for (i=0; i<NUM_POSSIBLE_STATES; ++i)
    {
      if (!strcmp(name, state_names[i])) return GACCESSIBLE_STATE(i);
    }
  return 0;
}

/**
 * Return a reference to an object's Gaccessible implementation, if
 * the object implements GaccessibleInterface.
 * @object: The GObject instance which should implement #GaccessibleInterface
 * if a non-null return value is required.
 */
Gaccessible *
gaccessible_object_ref_accessible (GObject *object)
{
  GaccessibleInterface *iface;
  Gaccessible          *accessible = NULL;

  g_return_val_if_fail ((object != NULL), NULL);
  g_return_val_if_fail ((iface = GACCESSIBLE_GET_INTERFACE (object)), NULL );

  if (iface != NULL) accessible =  (*(iface->ref_accessible)) (object) ;

  g_return_val_if_fail ((accessible != NULL), NULL);

  return GACCESSIBLE (accessible) ;
}

GaccessibleActionInterface*
gaccessible_get_action_interface (Gaccessible *accessible)
{
  g_return_val_if_fail ((accessible != NULL), NULL);
  g_return_val_if_fail (G_IS_GACCESSIBLE (accessible), NULL);
  return GACCESSIBLE_ACTION_GET_INTERFACE(accessible);
}

GaccessibleComponentInterface*
gaccessible_get_component_interface (Gaccessible *accessible)
{
  g_return_val_if_fail ((accessible != NULL), NULL);
  g_return_val_if_fail (G_IS_GACCESSIBLE (accessible), NULL);
  return GACCESSIBLE_COMPONENT_GET_INTERFACE(accessible);
}

GaccessibleEditableTextInterface*
gaccessible_get_editable_text_interface (Gaccessible *accessible)
{
  g_return_val_if_fail ((accessible != NULL), NULL);
  g_return_val_if_fail (G_IS_GACCESSIBLE (accessible), NULL);
  return GACCESSIBLE_EDITABLE_TEXT_GET_INTERFACE(accessible);
}

GaccessibleHypertextInterface*
gaccessible_get_hypertext_interface (Gaccessible *accessible)
{
  g_return_val_if_fail ((accessible != NULL), NULL);
  g_return_val_if_fail (G_IS_GACCESSIBLE (accessible), NULL);
  return GACCESSIBLE_HYPERTEXT_GET_INTERFACE(accessible);
}

GaccessibleIconInterface*
gaccessible_get_icon_interface (Gaccessible *accessible)
{
  g_return_val_if_fail ((accessible != NULL), NULL);
  g_return_val_if_fail (G_IS_GACCESSIBLE (accessible), NULL);
  return GACCESSIBLE_ICON_GET_INTERFACE(accessible);
}

GaccessibleSelectionInterface*
gaccessible_get_selection_interface (Gaccessible *accessible)
{
  g_return_val_if_fail ((accessible != NULL), NULL);
  g_return_val_if_fail (G_IS_GACCESSIBLE (accessible), NULL);
  return GACCESSIBLE_SELECTION_GET_INTERFACE(accessible);
}

GaccessibleTableInterface*
gaccessible_get_table_interface (Gaccessible *accessible)
{
  g_return_val_if_fail ((accessible != NULL), NULL);
  g_return_val_if_fail (G_IS_GACCESSIBLE (accessible), NULL);
  return GACCESSIBLE_TABLE_GET_INTERFACE(accessible);
}

GaccessibleTextInterface*
gaccessible_get_text_interface (Gaccessible *accessible)
{
  g_return_val_if_fail ((accessible != NULL), NULL);
  g_return_val_if_fail (G_IS_GACCESSIBLE (accessible), NULL);
  return GACCESSIBLE_TEXT_GET_INTERFACE(accessible);
}

GaccessibleValueInterface*
gaccessible_get_value_interface (Gaccessible *accessible)
{
  g_return_val_if_fail ((accessible != NULL), NULL);
  g_return_val_if_fail (G_IS_GACCESSIBLE (accessible), NULL);
  return GACCESSIBLE_VALUE_GET_INTERFACE(accessible);
}

GType
gaccessible_component_get_type ()
{
  static GType type = 0;

  if (!type) {
    GTypeInfo tinfo =
    {
      sizeof (GaccessibleComponentInterface),
      NULL,
      NULL,

    };

    type = g_type_register_static (G_TYPE_INTERFACE, "gaccessible_component", &tinfo
, 0);
  }

  return type;
}

GType
gaccessible_selection_get_type ()
{
  static GType type = 0;

  if (!type) {
    GTypeInfo tinfo =
    {
      sizeof (GaccessibleSelectionInterface),
      NULL,
      NULL,

    };

    type = g_type_register_static (G_TYPE_INTERFACE, "gaccessible_selection", &tinfo
, 0);
  }

  return type;
}

GType
gaccessible_hypertext_get_type ()
{
  static GType type = 0;

  if (!type) {
    GTypeInfo tinfo =
    {
      sizeof (GaccessibleHypertextInterface),
      NULL,
      NULL,

    };

    type = g_type_register_static (G_TYPE_INTERFACE, "gaccessible_hypertext", &tinfo
, 0);
  }

  return type;
}

GType
gaccessible_action_get_type ()
{
  static GType type = 0;

  if (!type) {
    GTypeInfo tinfo =
    {
      sizeof (GaccessibleActionInterface),
      NULL,
      NULL,

    };

    type = g_type_register_static (G_TYPE_INTERFACE, "gaccessible_action", &tinfo
, 0);
  }

  return type;
}

GType
gaccessible_icon_get_type ()
{
  static GType type = 0;

  if (!type) {
    GTypeInfo tinfo =
    {
      sizeof (GaccessibleIconInterface),
      NULL,
      NULL,

    };

    type = g_type_register_static (G_TYPE_INTERFACE, "gaccessible_icon", &tinfo
, 0);
  }

  return type;
}

GType
gaccessible_editable_text_get_type ()
{
  static GType type = 0;

  if (!type) {
    GTypeInfo tinfo =
    {
      sizeof (GaccessibleEditableTextInterface),
      NULL,
      NULL,

    };

    type = g_type_register_static (G_TYPE_INTERFACE, "gaccessible_editable_text", &tinfo
, 0);
  }

  return type;
}

GType
gaccessible_text_get_type ()
{
  static GType type = 0;

  if (!type) {
    GTypeInfo tinfo =
    {
      sizeof (GaccessibleTextInterface),
      NULL,
      NULL,

    };

    type = g_type_register_static (G_TYPE_INTERFACE, "gaccessible_text", &tinfo
, 0);
  }

  return type;
}

GType
gaccessible_table_get_type ()
{
  static GType type = 0;

  if (!type) {
    GTypeInfo tinfo =
    {
      sizeof (GaccessibleTableInterface),
      NULL,
      NULL,

    };

    type = g_type_register_static (G_TYPE_INTERFACE, "gaccessible_table", &tinfo
, 0);
  }

  return type;
}

