/* GTK - The GIMP Toolkit
 * Copyright (C) 1995-1997 Peter Mattis, Spencer Kimball and Josh MacDonald
 *
 * This library is free software; you can redistribute it and/or
 * modify it under the terms of the GNU Library General Public
 * License as published by the Free Software Foundation; either
 * version 2 of the License, or (at your option) any later version.
 *
 * This library is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 * Library General Public License for more details.
 *
 * You should have received a copy of the GNU Library General Public
 * License along with this library; if not, write to the
 * Free Software Foundation, Inc., 59 Temple Place - Suite 330,
 * Boston, MA 02111-1307, USA.
 */

/*
 * Modified by the GTK+ Team and others 1997-2001.  See the AUTHORS
 * file for a list of people on the GTK+ Team.  See the ChangeLog
 * files for a list of changes.  These files are distributed with
 * GTK+ at ftp://ftp.gtk.org/pub/gtk/.
 */

#ifndef __G_ACCESSIBLE_H__
#define __G_ACCESSIBLE_H__

#ifdef __cplusplus
extern "C" {
#endif /* __cplusplus */

#include <gobject/gobject.h>
#include <gobject/gclosure.h>
#include <gobject/gmarshal.h>

typedef enum
{
  ROLE_ALERT,
  ROLE_CHECK_BOX,
  ROLE_COLOR_CHOOSER,
  ROLE_COLUMN_HEADER,
  ROLE_COMBO_BOX,
  ROLE_DESKTOP_ICON,
  ROLE_DESKTOP_FRAME,
  ROLE_DIALOG,
  ROLE_DIRECTORY_PANE,
  ROLE_FILE_CHOOSER,
  ROLE_FILLER,
  ROLE_FOCUS_TRAVERSABLE,
  ROLE_FRAME,
  ROLE_GLASS_PANE,
  ROLE_ICON,
  ROLE_INTERNAL_FRAME,
  ROLE_LABEL,
  ROLE_LAYERED_PANE,
  ROLE_LIST,
  ROLE_LIST_ITEM,
  ROLE_MENU,
  ROLE_MENU_BAR,
  ROLE_MENU_ITEM,
  ROLE_OPTION_PANE,
  ROLE_PAGE_TAB,
  ROLE_PAGE_TAB_LIST,
  ROLE_PANEL,
  /* ROLE_PASSWD_TEXT */
  ROLE_POPUP_MENU,
  ROLE_PROGRESS_BAR,
  ROLE_PUSH_BUTTON,
  ROLE_RADIO_BUTTON,
  ROLE_ROOT_PANE,
  ROLE_ROW_HEADER,
  ROLE_SCROLL_BAR,
  ROLE_SCROLL_PANE,
  ROLE_SEPARATOR,
  ROLE_SLIDER,
  ROLE_SPLIT_PANE,
  ROLE_TABLE,
  ROLE_TABLE_CELL,
  ROLE_TABLE_COLUMN_HEADER,
  ROLE_TABLE_ROW_HEADER,
  ROLE_TEXT,
  ROLE_TOGGLE_BUTTON,
  ROLE_TOOL_BAR,
  ROLE_TOOL_TIP,
  ROLE_TREE,
  ROLE_UNKNOWN,
  ROLE_VIEWPORT,
  ROLE_WINDOW
} GaccessibleRole;

typedef enum
{
  STATE_ACTIVE,
  STATE_ARMED,
  STATE_BUSY,
  STATE_CHECKED,
  STATE_COLLAPSED,
  STATE_EDITABLE,
  STATE_ENABLED,
  STATE_EXPANDABLE,
  STATE_EXPANDED,
  STATE_FOCUSABLE,
  STATE_FOCUSED,
  STATE_HORIZONTAL,
  STATE_ICONIFIED,
  STATE_MODAL,
  STATE_MULTI_LINE,
  STATE_MULTISELECTABLE,
  STATE_OPAQUE,
  STATE_PRESSED,
  STATE_RESIZABLE,
  STATE_SELECTABLE,
  STATE_SELECTED,
  STATE_SHOWING,
  STATE_SINGLE_LINE,
  STATE_TRANSIENT,
  STATE_VERTICAL,
  STATE_VISIBLE
} GaccessibleStateEnum;


#define TYPE_GACCESSIBLE                          (gaccessible_get_type ())
#define GACCESSIBLE(obj)                          (G_TYPE_CHECK_INSTANCE_CAST ((obj), TYPE_GACCESSIBLE, Gaccessible))
#define GACCESSIBLE_CLASS(klass)                  (G_TYPE_CHECK_CLASS_CAST ((klass), TYPE_GACCESSIBLE, GaccessibleClass))
#define G_IS_GACCESSIBLE(obj)                     (G_TYPE_CHECK_INSTANCE_TYPE ((obj), TYPE_GACCESSIBLE))
#define G_IS_GACCESSIBLE_CLASS(klass)             (G_TYPE_CHECK_CLASS_TYPE ((klass), TYPE_GACCESSIBLE))
#define GACCESSIBLE_GET_CLASS(obj)                (G_TYPE_INSTANCE_GET_CLASS ((obj), TYPE_GACCESSIBLE, GaccessibleClass))

#define TYPE_GACCESSIBLE_INTERFACE                (gaccessible_interface_get_type ())
#define GACCESSIBLE_GET_INTERFACE(obj)            (G_TYPE_INSTANCE_GET_INTERFACE ((obj), TYPE_GACCESSIBLE_INTERFACE, GaccessibleInterface))

/* Forward declarations of interface structures */

typedef struct _GaccessibleInterface            GaccessibleInterface;

typedef struct _GaccessibleActionInterface      GaccessibleActionInterface;
typedef struct _GaccessibleComponentInterface   GaccessibleComponentInterface;
typedef struct _GaccessibleEditableTextInterface GaccessibleEditableTextInterface;
typedef struct _GaccessibleHypertextInterface   GaccessibleHypertextInterface;
typedef struct _GaccessibleIconInterface        GaccessibleIconInterface;
typedef struct _GaccessibleSelectionInterface   GaccessibleSelectionInterface;
typedef struct _GaccessibleTableInterface       GaccessibleTableInterface;
typedef struct _GaccessibleTextInterface        GaccessibleTextInterface;
typedef struct _GaccessibleValueInterface       GaccessibleValueInterface;


typedef struct _Gaccessible                     Gaccessible;
typedef struct _GaccessibleClass                GaccessibleClass;
typedef struct _GaccessibleRelation             GaccessibleRelation;
typedef struct _GaccessibleRelationSet          GaccessibleRelationSet;

typedef guint32                                 GaccessibleStateMask;
typedef guint32                                 GaccessibleState;

#define GACCESSIBLE_STATE(state_enum)           ((GaccessibleStateMask)(1 << ((guint32)(state_enum)%32)))
#define NUM_POSSIBLE_STATES                     (sizeof(GaccessibleStateMask)*8)


typedef GClosure                                GaccessiblePropertyChangeHandler;

struct _Gaccessible
{
  GObject _parent;

  gchar *description;
  gchar *name;
  Gaccessible *parent;
  GaccessibleRole role;
  GaccessibleState *state;
};

struct _GaccessibleClass
{
  GObjectClass _parent;

  guint        (* get_child_count)     (Gaccessible *accessible);
  Gaccessible* (* ref_child)           (Gaccessible *accessible,
                                        guint       i);
  guint        (* get_index_in_parent) (Gaccessible *accessible,
                                        guint       i);

  /* signals */
  void         (* children_changed)    (Gaccessible *accessible,
                                        guint       change_type,
                                        guint       child_index,
                                        Gaccessible *changed_child);
};
GType            gaccessible_get_type   (void);

struct _GaccessibleInterface
{
  GTypeInterface parent;

  Gaccessible* (*ref_accessible) (GObject *object);
};
GType gaccessible_interface_get_type (void);

/*
 * This method uses the ref_accessible method in GaccessibleInterface,
 * if the object's class implements GaccessibleInterface.
 * Otherwise it returns %NULL.
 *
 * IMPORTANT:
 * Note also that because this method may return flyweight objects,
 * it increments the returned Gaccessible's reference count.
 * Therefore it is the responsibility of the calling
 * program to unreference the object when no longer needed.
 * (c.f. gtk_widget_get_accessible() where this is not the case).
 */
Gaccessible* gaccessible_object_ref_accessible (GObject *object);

/*
 * Properties directly supported by Gaccessible
 */

gchar*                  gaccessible_get_name                      (Gaccessible *accessible);
gchar*                  gaccessible_get_description               (Gaccessible *accessible);
Gaccessible*            gaccessible_get_parent                    (Gaccessible *accessible);
guint                   gaccessible_get_accessible_children_count (Gaccessible *accessible);
Gaccessible*            gaccessible_ref_accessible_child          (Gaccessible *accessible,
                                                                   guint       i);
GaccessibleRelationSet* gaccessible_get_relation_set              (Gaccessible *accessible);
GaccessibleRole         gaccessible_get_role                      (Gaccessible *accessible);
GaccessibleState*       gaccessible_get_state                     (Gaccessible *accessible);
guint                   gaccessible_get_index_in_parent           (Gaccessible *accessible);
void                    gaccessible_set_name                      (Gaccessible *accessible,
                                                                   const gchar *name);
void                    gaccessible_set_description               (Gaccessible *accessible,
                                                                   const gchar *description);
void                    gaccessible_set_parent                    (Gaccessible *accessible,
                                                                   Gaccessible *parent);


/*
 * to install property change listener, one must
 * install signal handler for gobject "properties_changed" signal.
 * (for single notifications of multiple changes).
 * We could use the "notify" signal instead.
 *
 * Note: we have not yet determined exactly what the following
 * two convenience function signatures should look like.
 */
guint                gaccessible_connect_property_change_handler (Gaccessible                      *accessible,
                                                                  GaccessiblePropertyChangeHandler *handler);
void                 gaccessible_remove_property_change_handler  (Gaccessible                      *accessible,
                                                                  guint                            handler_id);

/*
 * Note: the properties which are registered with the GType
 *   property registry, for type Gaccessible, are as follows:
 *
 *   "accessible_name"
 *   "accessible_description"
 *   "accessible_parent"
 *   "accessible_child"
 *   "accessible_role"
 *   "accessible_state"
 *
 * accessibility property change listeners should use the
 *   normal GObject property interfaces and "properties_changed"
 *   signal handler semantics to interpret the property change
 *   information relayed passed from Gaccessible.
 *   (Gaccessible instances will connect to the "properties_changed"
 *   signal in their host objects, and relay the signals when appropriate).
 */

/* For other properties, see related interfaces
 *
 *    GaccessibleActionInterface,
 *    GaccessibleComponentInterface,
 *    GaccessibleHypertextInterface,
 *    GaccessibleIconInterface,
 *    GaccessibleSelectionInterface,
 *    GaccessibleTableInterface,
 *    GaccessibleTextInterface,
 *    GaccessibleValueInterface.
 *
 *  The usage model for obtaining these interface instances is:
 *    GACCESSIBLE_<interfacename>_GET_INTERFACE(GObject *accessible),
 *    where accessible, though specified as a GObject, is
 *    the Gaccessible instance being queried.
 *  There may be additional macros, thus:
 *    if (GACCESSIBLE_TEXT_GET_INTERFACE(accessible))
 *      gint pos = GACCESSIBLE_TEXT_GET_CARET_POSITION(accessible);
 *
 *  or more efficiently:
 *
 *    GaccessibleTextInterface iface
 *      = GACCESSIBLE_TEXT_GET_INTERFACE(accessible);
 *    if (iface)
 *      GACCESSIBLE_TEXT_INTERFACE_INSTANCE_GET_CARET_POSITION(if, accessible);
 */

/*
 * The following methods are provided for convenience only, their
 * implementations are identical to the use model detailed above.
 */
GaccessibleActionInterface*       gaccessible_get_action_interface        (Gaccessible *accessible);
GaccessibleComponentInterface*    gaccessible_get_component_interface     (Gaccessible *accessible);
GaccessibleEditableTextInterface *gaccessible_get_editable_text_interface (Gaccessible *accessible);
GaccessibleHypertextInterface*    gaccessible_get_hypertext_interface     (Gaccessible *accessible);
GaccessibleIconInterface*         gaccessible_get_icon_interface          (Gaccessible *accessible);
GaccessibleSelectionInterface*    gaccessible_get_selection_interface     (Gaccessible *accessible);
GaccessibleTableInterface*        gaccessible_get_table_interface         (Gaccessible *accessible);
GaccessibleTextInterface*         gaccessible_get_text_interface          (Gaccessible *accessible);
GaccessibleValueInterface*        gaccessible_get_value_interface         (Gaccessible *accessible);


enum
{
  RELATION_CONTROLLED_BY,
  RELATION_CONTROLLER_FOR,
  RELATION_LABEL_FOR,
  RELATION_LABELLED_BY,
  RELATION_MEMBER_OF
};

GaccessibleRelation* gaccessible_relation_new                         (GObject                *target,
                                                                       guint                   relationship);
void                 gaccessible_relation_set_add                     (GaccessibleRelationSet *set,
                                                                       GaccessibleRelation    *relation);
void                 gaccessible_relation_set_remove                  (GaccessibleRelationSet *set,
                                                                       GaccessibleRelation    *relation);
guint                gaccessible_relation_set_get_target_relationship (GaccessibleRelationSet *set,
                                                                       GObject                *target);
gchar*               gaccessible_state_mask_get_name                  (GaccessibleStateMask    state);
GaccessibleStateMask gaccessible_state_mask_for_name                  (gchar                  *name);


#ifdef __cplusplus
}
#endif /* __cplusplus */


#endif /* __G_ACCESSIBLE_H__ */
