/* GTK - The GIMP Toolkit
 * Copyright (C) 1995-1997 Peter Mattis, Spencer Kimball and Josh MacDonald
 *
 * This library is free software; you can redistribute it and/or
 * modify it under the terms of the GNU Lesser General Public
 * License as published by the Free Software Foundation; either
 * version 2 of the License, or (at your option) any later version.
 *
 * This library is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 * Lesser General Public License for more details.
 *
 * You should have received a copy of the GNU Lesser General Public
 * License along with this library; if not, write to the
 * Free Software Foundation, Inc., 59 Temple Place - Suite 330,
 * Boston, MA 02111-1307, USA.
 */

/*
 * Modified by the GTK+ Team and others 1997-2000.  See the AUTHORS
 * file for a list of people on the GTK+ Team.  See the ChangeLog
 * files for a list of changes.  These files are distributed with
 * GTK+ at ftp://ftp.gtk.org/pub/gtk/.
 */

#include <stdarg.h>
#include <string.h>

#include "gaccessiblevalue.h"

GType
gaccessible_value_get_type ()
{
  static GType type = 0;

  if (!type) {
    GTypeInfo tinfo =
    {
      sizeof (GaccessibleValueInterface),
      NULL,
      NULL,

    };

    type = g_type_register_static (G_TYPE_INTERFACE, "gaccessible_value", &tinfo, 0);
  }

  return type;
}

/**
 * gaccessible_value_get_current_value:
 * @value: a GObject instance that implements GaccessibleValueInterface
 * @return: a #gdouble representing the current accessible value, or %NULL
 * if value does not implement this interface.
 *
 * WARNING: callers should not rely on %NULL or on a zero value for
 * indication of whether GaccessibleValue is implemented, they should
 * use type checking/interface checking macros or the
 * gaccessible_get_accessible_value() convenience method.
 **/
gdouble
gaccessible_value_get_current_value (GaccessibleValue *obj)
{
  GaccessibleValueInterface *iface;
  gdouble dzero = 0.0;

  g_return_val_if_fail (obj != NULL, dzero);
  g_return_val_if_fail (IS_GACCESSIBLE_VALUE (obj), dzero);
  g_return_val_if_fail (iface = GACCESSIBLE_VALUE_GET_INTERFACE (obj), dzero);

  if (iface->get_current_value)
    return (iface->get_current_value) (obj);
  else
    return dzero;
}

/**
 * gaccessible_value_get_maximum_value:
 * @value: a GObject instance that implements GaccessibleValueInterface
 * @return: a #gdouble representing the maximum accessible value, or %NULL
 * if value does not implement this interface.
 *
 * WARNING: callers should not rely on %NULL or on a zero value for
 * indication of whether GaccessibleValue is implemented, they should
 * use type checking/interface checking macros or the
 * gaccessible_get_accessible_value() convenience method.
 **/
gdouble
gaccessible_value_get_maximum_value  (GaccessibleValue *obj)
{
  GaccessibleValueInterface *iface;
  gdouble dzero = 0.0;

  g_return_val_if_fail (obj != NULL, dzero);
  g_return_val_if_fail (IS_GACCESSIBLE_VALUE (obj), dzero);
  iface = GACCESSIBLE_VALUE_GET_INTERFACE (obj);
  g_return_val_if_fail (iface, dzero);

  if (iface->get_maximum_value)
    return (iface->get_maximum_value) (obj);
  else
    return dzero;
}

/**
 * gaccessible_value_get_minimum_value:
 * @value: a GObject instance that implements GaccessibleValueInterface
 * @return: a #gdouble representing the minimum accessible value, or %NULL
 * if value does not implement this interface.
 *
 * WARNING: callers should not rely on %NULL or on a zero value for
 * indication of whether GaccessibleValue is implemented, they should
 * use type checking/interface checking macros or the
 * gaccessible_get_accessible_value() convenience method.
 **/
gdouble
gaccessible_value_get_minimum_value (GaccessibleValue *obj)
{
  GaccessibleValueInterface *iface;
  gdouble dzero = 0.0;

  g_return_val_if_fail (obj != NULL, dzero);
  g_return_val_if_fail (IS_GACCESSIBLE_VALUE (obj), dzero);
  g_return_val_if_fail (iface = GACCESSIBLE_VALUE_GET_INTERFACE (obj), dzero);

  if (iface->get_minimum_value)
    return (iface->get_minimum_value) (obj);
  else
    return dzero;
}

/**
 * gaccessible_value_set_current_value:
 * @value: a GObject instance that implements GaccessibleValueInterface
 * @dval: a gdouble which is the desired new accessible value.
 * @return: %true if new value is successfully set, %false otherwise.
 **/
gboolean
gaccessible_value_set_current_value (GaccessibleValue *obj, gdouble dval)
{
  GaccessibleValueInterface *iface;

  g_return_val_if_fail (obj != NULL, FALSE);
  g_return_val_if_fail (IS_GACCESSIBLE_VALUE (obj), FALSE);
  g_return_val_if_fail (iface = GACCESSIBLE_VALUE_GET_INTERFACE (obj), FALSE);

  if (iface->set_current_value)
    return (iface->set_current_value) (obj, dval);
  else
    return FALSE;
}




